@@ -1,13 +1,12 @@
 use crate::lang::SgLang;
 use ast_grep_config::{RuleConfig, Severity};
-use ast_grep_core::{meta_var::MetaVariable, Node as SgNode, NodeMatch as SgNodeMatch, StrDoc};
+use ast_grep_core::{meta_var::MetaVariable, Node as SgNode, StrDoc};
 
-type NodeMatch<'a, L> = SgNodeMatch<'a, StrDoc<L>>;
 type Node<'a, L> = SgNode<'a, StrDoc<L>>;
 
 use std::collections::HashMap;
 
-use super::{Diff, Printer};
+use super::{Diff, NodeMatch, PrintProcessor, Printer};
 use anyhow::Result;
 use clap::ValueEnum;
 use codespan_reporting::files::SimpleFile;
@@ -17,14 +16,6 @@ use std::borrow::Cow;
 use std::io::{Stdout, Write};
 use std::path::Path;
 
-// add this macro because neither trait_alias nor type_alias_impl is supported.
-macro_rules! Matches {
-  ($lt: lifetime) => { impl Iterator<Item = NodeMatch<$lt, SgLang>> };
-}
-macro_rules! Diffs {
-  ($lt: lifetime) => { impl Iterator<Item = Diff<$lt>> };
-}
-
 #[derive(Serialize, Deserialize)]
 #[serde(rename_all = "camelCase")]
 /// Zero-based character position in a file.
@@ -77,7 +68,7 @@ struct MatchJSON<'a> {
   lines: String,
   char_count: CharCount,
   #[serde(skip_serializing_if = "Option::is_none")]
-  replacement: Option<Cow<'a, str>>,
+  replacement: Option<String>,
   #[serde(skip_serializing_if = "Option::is_none")]
   replacement_offsets: Option<std::ops::Range<usize>>,
   language: SgLang,
@@ -92,7 +83,7 @@ struct MetaVariables<'a> {
   multi: HashMap<String, Vec<MatchNode<'a>>>,
   transformed: HashMap<String, String>,
 }
-fn from_env<'a>(nm: &NodeMatch<'a, SgLang>) -> Option<MetaVariables<'a>> {
+fn from_env<'a>(nm: &NodeMatch<'a>) -> Option<MetaVariables<'a>> {
   let env = nm.get_env();
   let mut vars = env.get_matched_variables().peekable();
   vars.peek()?;
@@ -155,7 +146,7 @@ fn get_range(n: &Node<'_, SgLang>) -> Range {
 }
 
 impl<'a> MatchJSON<'a> {
-  fn new(nm: NodeMatch<'a, SgLang>, path: &'a str, context: (u16, u16)) -> Self {
+  fn new(nm: NodeMatch<'a>, path: &'a str, context: (u16, u16)) -> Self {
     let display = nm.display_context(context.0 as usize, context.1 as usize);
     let lines = format!("{}{}{}", display.leading, display.matched, display.trailing);
     MatchJSON {
@@ -181,7 +172,7 @@ impl<'a> MatchJSON<'a> {
     ret
   }
 }
-fn get_labels<'a>(nm: &NodeMatch<'a, SgLang>) -> Option<Vec<MatchNode<'a>>> {
+fn get_labels<'a>(nm: &NodeMatch<'a>) -> Option<Vec<MatchNode<'a>>> {
   let env = nm.get_env();
   let labels = env.get_labels("secondary")?;
   Some(
@@ -208,7 +199,7 @@ struct RuleMatchJSON<'a> {
   labels: Option<Vec<MatchNode<'a>>>,
 }
 impl<'a> RuleMatchJSON<'a> {
-  fn new(nm: NodeMatch<'a, SgLang>, path: &'a str, rule: &'a RuleConfig<SgLang>) -> Self {
+  fn new(nm: NodeMatch<'a>, path: &'a str, rule: &'a RuleConfig<SgLang>) -> Self {
     let message = rule.get_message(&nm);
     let labels = get_labels(&nm);
     let matched = MatchJSON::new(nm, path, (0, 0));
@@ -280,85 +271,39 @@ impl<W: Write> JSONPrinter<W> {
     self.context = context;
     self
   }
-
-  fn print_docs<S: Serialize>(&mut self, mut docs: impl Iterator<Item = S>) -> Result<()> {
-    let Some(doc) = docs.next() else {
-      return Ok(());
-    };
-    let output = &mut self.output;
-    let matched = self.matched;
-    self.matched = true;
-    match self.style {
-      JsonStyle::Pretty => {
-        if matched {
-          writeln!(output, ",")?;
-        } else {
-          writeln!(output)?;
-        }
-        serde_json::to_writer_pretty(&mut *output, &doc)?;
-        for doc in docs {
-          writeln!(output, ",")?;
-          serde_json::to_writer_pretty(&mut *output, &doc)?;
-        }
-      }
-      JsonStyle::Stream => {
-        serde_json::to_writer(&mut *output, &doc)?;
-        writeln!(output)?;
-        for doc in docs {
-          serde_json::to_writer(&mut *output, &doc)?;
-          writeln!(output)?;
-        }
-      }
-      JsonStyle::Compact => {
-        if matched {
-          write!(output, ",")?;
-        }
-        serde_json::to_writer(&mut *output, &doc)?;
-        for doc in docs {
-          write!(output, ",")?;
-          serde_json::to_writer(&mut *output, &doc)?;
-        }
-      }
-    }
-    Ok(())
-  }
 }
 
 impl<W: Write> Printer for JSONPrinter<W> {
-  fn print_rule<'a>(
-    &mut self,
-    matches: Matches!('a),
-    file: SimpleFile<Cow<str>, &String>,
-    rule: &RuleConfig<SgLang>,
-  ) -> Result<()> {
-    let path = file.name();
-    let jsons = matches.map(|nm| RuleMatchJSON::new(nm, path, rule));
-    self.print_docs(jsons)
-  }
+  type Processed = Buffer;
+  type Processor = JSONProcessor;
 
-  fn print_matches<'a>(&mut self, matches: Matches!('a), path: &Path) -> Result<()> {
-    let path = path.to_string_lossy();
-    let context = self.context;
-    let jsons = matches.map(|nm| MatchJSON::new(nm, &path, context));
-    self.print_docs(jsons)
+  fn get_processor(&self) -> JSONProcessor {
+    JSONProcessor {
+      style: self.style,
+      context: self.context,
+    }
   }
-
-  fn print_diffs<'a>(&mut self, diffs: Diffs!('a), path: &Path) -> Result<()> {
-    let path = path.to_string_lossy();
-    let context = self.context;
-    let jsons = diffs.map(|diff| MatchJSON::diff(diff, &path, context));
-    self.print_docs(jsons)
-  }
-  fn print_rule_diffs(
-    &mut self,
-    diffs: Vec<(Diff<'_>, &RuleConfig<SgLang>)>,
-    path: &Path,
-  ) -> Result<()> {
-    let path = path.to_string_lossy();
-    let jsons = diffs
-      .into_iter()
-      .map(|(diff, rule)| RuleMatchJSON::diff(diff, &path, rule));
-    self.print_docs(jsons)
+  fn process(&mut self, processed: Buffer) -> Result<()> {
+    if processed.is_empty() {
+      return Ok(());
+    }
+    let output = &mut self.output;
+    let matched = self.matched;
+    self.matched = true;
+    // print separator if there was a match before
+    if matched {
+      let separator = match self.style {
+        JsonStyle::Pretty => ",\n",
+        JsonStyle::Stream => "\n",
+        JsonStyle::Compact => ",",
+      };
+      write!(output, "{separator}")?;
+    } else if self.style == JsonStyle::Pretty {
+      // print newline for the first match in pretty style
+      writeln!(output)?;
+    }
+    output.write_all(&processed)?;
+    Ok(())
   }
 
   fn before_print(&mut self) -> Result<()> {
@@ -382,6 +327,91 @@ impl<W: Write> Printer for JSONPrinter<W> {
   }
 }
 
+pub struct JSONProcessor {
+  style: JsonStyle,
+  context: (u16, u16),
+}
+
+impl JSONProcessor {
+  fn print_docs<S: Serialize>(&self, mut docs: impl Iterator<Item = S>) -> Result<Buffer> {
+    let mut ret = Vec::new();
+    let Some(doc) = docs.next() else {
+      return Ok(ret);
+    };
+    let output = &mut ret;
+    match self.style {
+      JsonStyle::Pretty => {
+        serde_json::to_writer_pretty(&mut *output, &doc)?;
+        for doc in docs {
+          writeln!(&mut *output, ",")?;
+          serde_json::to_writer_pretty(&mut *output, &doc)?;
+        }
+      }
+      JsonStyle::Stream => {
+        serde_json::to_writer(&mut *output, &doc)?;
+        for doc in docs {
+          writeln!(&mut *output)?;
+          serde_json::to_writer(&mut *output, &doc)?;
+        }
+      }
+      JsonStyle::Compact => {
+        serde_json::to_writer(&mut *output, &doc)?;
+        for doc in docs {
+          write!(output, ",")?;
+          serde_json::to_writer(&mut *output, &doc)?;
+        }
+      }
+    }
+    Ok(ret)
+  }
+}
+
+type Buffer = Vec<u8>;
+
+impl PrintProcessor<Buffer> for JSONProcessor {
+  fn print_rule(
+    &self,
+    matches: Vec<NodeMatch>,
+    file: SimpleFile<Cow<str>, &String>,
+    rule: &RuleConfig<SgLang>,
+  ) -> Result<Buffer> {
+    let path = file.name();
+    let jsons = matches
+      .into_iter()
+      .map(|nm| RuleMatchJSON::new(nm, path, rule));
+    self.print_docs(jsons)
+  }
+
+  fn print_matches(&self, matches: Vec<NodeMatch>, path: &Path) -> Result<Buffer> {
+    let path = path.to_string_lossy();
+    let context = self.context;
+    let jsons = matches
+      .into_iter()
+      .map(|nm| MatchJSON::new(nm, &path, context));
+    self.print_docs(jsons)
+  }
+
+  fn print_diffs(&self, diffs: Vec<Diff>, path: &Path) -> Result<Buffer> {
+    let path = path.to_string_lossy();
+    let context = self.context;
+    let jsons = diffs
+      .into_iter()
+      .map(|diff| MatchJSON::diff(diff, &path, context));
+    self.print_docs(jsons)
+  }
+  fn print_rule_diffs(
+    &self,
+    diffs: Vec<(Diff, &RuleConfig<SgLang>)>,
+    path: &Path,
+  ) -> Result<Buffer> {
+    let path = path.to_string_lossy();
+    let jsons = diffs
+      .into_iter()
+      .map(|(diff, rule)| RuleMatchJSON::diff(diff, &path, rule));
+    self.print_docs(jsons)
+  }
+}
+
 #[cfg(test)]
 mod test {
   use super::*;
@@ -412,9 +442,11 @@ mod test {
     for style in [JsonStyle::Pretty, JsonStyle::Compact] {
       let mut printer = make_test_printer(style);
       printer.before_print().unwrap();
-      printer
-        .print_matches(std::iter::empty(), "test.tsx".as_ref())
+      let buffer = printer
+        .get_processor()
+        .print_matches(vec![], "test.tsx".as_ref())
         .unwrap();
+      printer.process(buffer).unwrap();
       printer.after_print().unwrap();
       assert_eq!(get_text(&printer), "[]\n");
     }
@@ -451,9 +483,13 @@ mod test {
       // heading is required for CI
       let mut printer = make_test_printer(JsonStyle::Pretty);
       let grep = SgLang::from(SupportLang::Tsx).ast_grep(source);
-      let matches = grep.root().find_all(pattern);
+      let matches = grep.root().find_all(pattern).collect();
       printer.before_print().unwrap();
-      printer.print_matches(matches, "test.tsx".as_ref()).unwrap();
+      let buffer = printer
+        .get_processor()
+        .print_matches(matches, "test.tsx".as_ref())
+        .unwrap();
+      printer.process(buffer).unwrap();
       printer.after_print().unwrap();
       let json_str = get_text(&printer);
       let json: Vec<MatchJSON> = serde_json::from_str(&json_str).unwrap();
@@ -470,9 +506,15 @@ mod test {
       let grep = lang.ast_grep(source);
       let matches = grep.root().find_all(pattern);
       let fixer = Fixer::from_str(replace, &lang).expect("should work");
-      let diffs = matches.map(|m| Diff::generate(m, &pattern, &fixer));
+      let diffs = matches
+        .map(|m| Diff::generate(m, &pattern, &fixer))
+        .collect();
       printer.before_print().unwrap();
-      printer.print_diffs(diffs, "test.tsx".as_ref()).unwrap();
+      let buffer = printer
+        .get_processor()
+        .print_diffs(diffs, "test.tsx".as_ref())
+        .unwrap();
+      printer.process(buffer).unwrap();
       printer.after_print().unwrap();
       let json_str = get_text(&printer);
       let json: Vec<MatchJSON> = serde_json::from_str(&json_str).unwrap();
@@ -512,10 +554,14 @@ rule:
       let mut printer = make_test_printer(JsonStyle::Pretty);
       let grep = SgLang::from(SupportLang::Tsx).ast_grep(&source);
       let rule = make_rule(pattern);
-      let matches = grep.root().find_all(&rule.matcher);
+      let matches = grep.root().find_all(&rule.matcher).collect();
       printer.before_print().unwrap();
       let file = SimpleFile::new(Cow::Borrowed("test.ts"), &source);
-      printer.print_rule(matches, file, &rule).unwrap();
+      let buffer = printer
+        .get_processor()
+        .print_rule(matches, file, &rule)
+        .unwrap();
+      printer.process(buffer).unwrap();
       printer.after_print().unwrap();
       let json_str = get_text(&printer);
       let json: Vec<RuleMatchJSON> = serde_json::from_str(&json_str).unwrap();
@@ -529,9 +575,13 @@ rule:
     let mut printer = make_test_printer(JsonStyle::Pretty);
     let lang = SgLang::from(SupportLang::Tsx);
     let grep = lang.ast_grep("console.log(123)");
-    let matches = grep.root().find_all("console.log($A)");
+    let matches = grep.root().find_all("console.log($A)").collect();
     printer.before_print().unwrap();
-    printer.print_matches(matches, "test.tsx".as_ref()).unwrap();
+    let buffer = printer
+      .get_processor()
+      .print_matches(matches, "test.tsx".as_ref())
+      .unwrap();
+    printer.process(buffer).unwrap();
     printer.after_print().unwrap();
     let json_str = get_text(&printer);
     let json: Vec<MatchJSON> = serde_json::from_str(&json_str).unwrap();
@@ -548,9 +598,13 @@ rule:
     let mut printer = make_test_printer(JsonStyle::Compact);
     let lang = SgLang::from(SupportLang::Tsx);
     let grep = lang.ast_grep("console.log(1, 2, 3)");
-    let matches = grep.root().find_all("console.log($$$A)");
+    let matches = grep.root().find_all("console.log($$$A)").collect();
     printer.before_print().unwrap();
-    printer.print_matches(matches, "test.tsx".as_ref()).unwrap();
+    let buffer = printer
+      .get_processor()
+      .print_matches(matches, "test.tsx".as_ref())
+      .unwrap();
+    printer.process(buffer).unwrap();
     printer.after_print().unwrap();
     let json_str = get_text(&printer);
     let json: Vec<MatchJSON> = serde_json::from_str(&json_str).unwrap();
@@ -565,9 +619,13 @@ rule:
     for &(source, pattern, _, note) in MATCHES_CASES {
       let mut printer = make_test_printer(JsonStyle::Stream);
       let grep = SgLang::from(SupportLang::Tsx).ast_grep(source);
-      let matches = grep.root().find_all(pattern);
+      let matches = grep.root().find_all(pattern).collect();
       printer.before_print().unwrap();
-      printer.print_matches(matches, "test.tsx".as_ref()).unwrap();
+      let buffer = printer
+        .get_processor()
+        .print_matches(matches, "test.tsx".as_ref())
+        .unwrap();
+      printer.process(buffer).unwrap();
       printer.after_print().unwrap();
       let json_str = get_text(&printer);
       let jsons: Vec<&str> = json_str.lines().collect();
@@ -594,9 +652,13 @@ transform:
     let mut printer = make_test_printer(JsonStyle::Compact);
     let rule = get_rule_config(&format!("pattern: console.log($A)\n{}", TRANSFORM_TEXT));
     let grep = SgLang::from(SupportLang::TypeScript).ast_grep("console.log(123)");
-    let matches = grep.root().find_all(&rule.matcher);
+    let matches = grep.root().find_all(&rule.matcher).collect();
     printer.before_print().unwrap();
-    printer.print_matches(matches, "test.tsx".as_ref()).unwrap();
+    let buffer = printer
+      .get_processor()
+      .print_matches(matches, "test.tsx".as_ref())
+      .unwrap();
+    printer.process(buffer).unwrap();
     printer.after_print().unwrap();
     let json_str = get_text(&printer);
     let json: Vec<MatchJSON> = serde_json::from_str(&json_str).unwrap();
