@@ -1,5 +1,5 @@
 use crate::language::Language;
-use crate::matcher::{FindAllNodes, Matcher, NodeMatch};
+use crate::matcher::{FindAllNodes, Matcher, MatcherExt, NodeMatch};
 use crate::replacer::Replacer;
 use crate::source::{perform_edit, Content, Edit as E, TSParseError};
 use crate::traversal::{Pre, Visitor};
@@ -24,11 +24,11 @@ pub struct Position {
 }
 
 impl Position {
-  fn new(line: u32, byte_column: u32, byte_offset: u32) -> Self {
+  fn new(line: usize, byte_column: usize, byte_offset: usize) -> Self {
     Self {
-      line: line as usize,
-      byte_column: byte_column as usize,
-      byte_offset: byte_offset as usize,
+      line,
+      byte_column,
+      byte_offset,
     }
   }
   pub fn line(&self) -> usize {
@@ -222,15 +222,15 @@ impl<'r, D: Doc> Node<'r, D> {
   /// Nodes' start position in terms of zero-based rows and columns.
   pub fn start_pos(&self) -> Position {
     let pos = self.inner.start_position();
-    let byte = self.inner.start_byte();
-    Position::new(pos.row(), pos.column(), byte)
+    let byte = self.inner.start_byte() as usize;
+    Position::new(pos.row() as usize, pos.column() as usize, byte)
   }
 
   /// Nodes' end position in terms of rows and columns.
   pub fn end_pos(&self) -> Position {
     let pos = self.inner.end_position();
-    let byte = self.inner.end_byte();
-    Position::new(pos.row(), pos.column(), byte)
+    let byte = self.inner.end_byte() as usize;
+    Position::new(pos.row() as usize, pos.column() as usize, byte)
   }
 
   pub fn text(&self) -> Cow<'r, str> {
@@ -383,21 +383,18 @@ impl<'r, D: Doc> Node<'r, D> {
   }
 
   pub fn field_children(&self, name: &str) -> impl Iterator<Item = Node<'r, D>> {
-    let field_id = self
-      .root
-      .lang()
-      .get_ts_language()
-      .field_id_for_name(name)
-      .unwrap_or(0);
+    let field_id = self.root.lang().get_ts_language().field_id_for_name(name);
     let root = self.root;
     let mut cursor = self.inner.walk();
     cursor.goto_first_child();
-    let mut done = false;
+    // if field_id is not found, iteration is done
+    let mut done = field_id.is_none();
+
     std::iter::from_fn(move || {
       if done {
         return None;
       }
-      while cursor.field_id() != Some(field_id) {
+      while cursor.field_id() != field_id {
         if !cursor.goto_next_sibling() {
           return None;
         }
